#ifndef	TWINSTAR_DBUS_H
#define	TWINSTAR_DBUS_H

#include <stdint.h>
#include <glib-object.h>

G_BEGIN_DECLS

#define	UNKNOWN_NODEID	(0)
#define	VALID_NODEID(n)	((n) != UNKNOWN_NODEID)

#define TWINSTAR_TYPE_DBUS         (twinstar_dbus_get_type ())
#define TWINSTAR_DBUS(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), TWINSTAR_TYPE_DBUS, TwinstarDBus))
#define TWINSTAR_DBUS_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), TWINSTAR_TYPE_DBUS, TwinstarDBusClass))
#define TWINSTAR_IS_DBUS(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), TWINSTAR_TYPE_DBUS))
#define TWINSTAR_IS_DBUS_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), TWINSTAR_TYPE_DBUS))
#define TWINSTAR_DBUS_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TWINSTAR_TYPE_DBUS, TwinstarDBusClass))

typedef struct TwinstarDBusPrivate TwinstarDBusPrivate;

typedef struct {
	GObject parent;
	TwinstarDBusPrivate *priv;
} TwinstarDBus;

typedef struct {
	GObjectClass             parent_class;
} TwinstarDBusClass;

typedef enum {
	TWINSTAR_DBUS_ERROR_GENERAL,
	TWINSTAR_DBUS_NUM_ERRORS
} TwinstarDBusError;


#define	TWINSTAR_DBUS_ERROR		(twinstar_dbus_error_quark ())
#define	TWINSTAR_DBUS_TYPE_ERROR	(twinstar_dbus_error_get_type ())

GQuark	twinstar_dbus_error_quark	(void);
GType	twinstar_dbus_error_get_type	(void);

struct twinstar_dbus_notify_ops {
	int (*send_message)(void *context, const char *msg);
	int (*keep_alive)(void *context, uint32_t nodeid);
	int (*preferred_master_was_set)(void *context, gboolean preferred_master);
	int (*frozen_master_was_set)(void *context, uint32_t nodeid);
	int (*take_over)(void *context);
	int (*astribank_hotplug)(void *context, gboolean added,
			const char *devpath,
			const char *hardware_id,
			const char *location);
	int (*astribanks_count)(void *context);
};

TwinstarDBus *twinstar_dbus_new(uint32_t local_nodeid, const struct twinstar_dbus_notify_ops *notify_ops, void *context);
gboolean twinstar_dbus_register(TwinstarDBus *twdbus);

gboolean twinstar_notify_new_master(TwinstarDBus *twdbus, uint32_t nodeid);
gboolean twinstar_notify_frozen_master(TwinstarDBus *twdbus, uint32_t frozen_master);
gboolean twinstar_notify_license(TwinstarDBus *twdbus, gboolean license_ok);
gboolean twinstar_notify_joined(TwinstarDBus *twdbus, uint32_t nodeid, size_t node_count);
gboolean twinstar_notify_left(TwinstarDBus *twdbus, uint32_t nodeid, size_t node_count);
gboolean twinstar_notify_keep_alive(TwinstarDBus *twdbus, uint32_t nodeid);
gboolean twinstar_notify_got_message(TwinstarDBus *twdbus, const gchar *msg);
gboolean twinstar_notify_astribank_hotplug(TwinstarDBus *twdbus,
	uint32_t nodeid,
	uint32_t count,
	gboolean added,
	const gchar *hardware_id,
	const gchar *location,
	const gchar *devpath);
gboolean twinstar_notify_runstep_begin(TwinstarDBus *twdbus,
	const gchar *stage_name,
	const gchar *stage_action);
gboolean twinstar_notify_runstep_end(TwinstarDBus *twdbus,
	const gchar *stage_name,
	const gchar *stage_action,
	int stage_status);
gboolean twinstar_notify_runsteps_done(TwinstarDBus *twdbus, guint errors);

gboolean twinstar_get_preferred_master(TwinstarDBus *twdbus);
uint32_t twinstar_get_frozen_master(TwinstarDBus *twdbus);
gboolean twinstar_get_license_status(TwinstarDBus *twdbus);

G_END_DECLS

#endif	/* TWINSTAR_DBUS_H */
